'****************************************************************
'*  Name    : DS1307-S2.inc                                      *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 15/03/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Driver for the DS1307 I2C clock Calendar          *
'*          : Some code has been added for compatibility        *
'*          : to use with external I2C eeprom for Logger        *
'****************************************************************


'-------------------------------------------------------------------------------
' Routines to access a DS1307 Real Time Clock/Calendar via a software I2C bus
' Written by L.Johnson 08-01-16
' Modified for the Hardware Busin/Busout by A. Freixanet 7/03/2016
' and could be compatible with the 24LC256.inc library
.'-------------------------------------------------------------------------------
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
$ifndef _DS1307_INC_
$define _DS1307_INC_
'
'-------------------------------------------------------------------------------    
    GoTo _DS1307_INCEXIT                  ' Jump over the routines
'-------------------------------------------------------------------------------
' Uncomment the next line to disable the NACK function.
'$define _DisableNACK_

' Uncomment the next line to enable the interrupts protection.
'$define _EnableGIE_Save_

' Default language.
$if (!_defined(_English_) && !_defined(_Spanish_) && !_defined(_Spanish_))
$define _English_
$endif
'-------------------------------------------------------------------------------
' Declare the default pins used for the I2Cin and I2COut commands.
'
#ifnSym __SDA_Port
    Declare SDA_Pin PORTC.4           ' SDA connected to this pin
#endIfSym
#ifnSym __SCL_Port
    Declare SCL_Pin PORTC.3           ' SCL connected to this pin
#endIfSym

'-------------------------------------------------------------------------------
' Add LOCAL (non-persistent) variables here
$ifndef _LVBS_Block1_
    $define _LVBS_Block1_
    Dim LVSByte1 As Byte System
    Dim LVSByte2 As Byte System
    Dim LVSByte3 As Byte System
    Dim LVSByte4 As Byte System
$endif
'-------------------------------------------------------------------------------
' Parameter variables
'
    Dim DS1307_bValue  As LVSByte3
    Dim DS1307_bResult As LVSByte4
'-------------------------------------------------------------------------------
' Time variables BCD & Decimal values
'
    Dim DS1307_Array[8] As Byte
    Dim RTC_Second    As DS1307_Array#0     ' Second (0..59)
    Dim RTC_Minute    As DS1307_Array#1     ' Minute (0..59)
    Dim RTC_Hour      As DS1307_Array#2     ' Hour    (0..11 or 0..23)
    Dim RTC_DayOfWeek As DS1307_Array#3     ' Day of the week (1..7)
    Dim RTC_Day       As DS1307_Array#4     ' Date  (0..31)
    Dim RTC_Month     As DS1307_Array#5     ' Month (1..12)
    Dim RTC_Year      As DS1307_Array#6     ' Year  (0..99)
'
    Dim RTC_DayName   As String * 9         ' Holds the name of the day
    Dim RTC_MonthName As String * 10        ' Holds the name of the month
'-------------------------------------------------------------------------------
' Date Variables used to read from the epprom or as locak variables.
' Use the "24LC256.inc" library downloaded from the WIKI of the PDS forum.
'
    Dim E2PArray[8]   As Byte 
    Dim vRTC_Second    As E2PArray#0   ' Second (0..59)
    Dim vRTC_Minute    As E2PArray#1   ' Minute (0..59)
    Dim vRTC_Hour      As E2PArray#2   ' Hour    (0..11 or 0..23)
    Dim vRTC_DayOfWeek As E2PArray#3   ' Day of the week (1..7)
    Dim vRTC_Day       As E2PArray#4   ' Date  (0..31)
    Dim vRTC_Month     As E2PArray#5   ' Month (1..12)
    Dim vRTC_Year      As E2PArray#6   ' Year  (0..99)
'-------------------------------------------------------------------------------
' DS1307 register addresses
'
     $define cRTC_DEV_ADDR  $D0         ' %11010000 DS1307 device Address
     $define cRTC_TIME      $00         ' %00000000 Address for time
     $define cRTC_WDAY      $03         ' %00000011 Address for day of week number
     $define cRTC_DATE      $04         ' %00000100 Address for date
     $define cRTC_CONTROL   $07         ' %00000111 Address for control byte
     $define cRTC_NVRAM     $08         ' %00001000 NVRAM start
     $define cRTC_NVRAM_MAX $38         ' %00111000 NVRAM address max (56 decimal)
'
' Values - used in calls to DS1307_ReadControl() and DS1307_WriteControl()...
'
     $define cRTC_SQWE       $10        ' %00010000 Square Wave enable
     $define cRTC_SQWF_1HZ   $00        ' %00000000 Square Wave frequency  1Hz
     $define cRTC_SQWF_4KHz  $01        ' %00000001 Square Wave frequency  4.096KHz
     $define cRTC_SQWF_8KHz  $02        ' %00000010 Square Wave frequency  8.192KHz
     $define cRTC_SQWF_32KHz $03        ' %00000011 Square Wave frequency 32.768KHz
     $define cRTC_SQWOUT_H   $80        ' %10000000 Set Square Wave output to high when disabled
'-------------------------------------------------------------------------------
$ifndef False
    $define False 0
$endif
$ifndef True
    $define True 1
$endif
'-------------------------------------------------------------------------------
' Values for Address Device to Read & Write
    Symbol xDS_DeviceRead = cRTC_DEV_ADDR | $01    
    Symbol xDS_DeviceWrite = cRTC_DEV_ADDR
'*******************************************************************************
$ifdef _EnableGIE_Save_
' Declare a variable for INTCON save.
    $ifndef _xINTCON_Save_Var_
    $define _xINTCON_Save_Var_
    Dim xINTCON_Save As Byte System
    $endif

'------------------------------------------------------------------------------- 
' This new command save and clear the GIE global interrupt.
$ifndef GIE_Save
    $define GIE_Save()          '              
    #ifdef __INTERRUPTS_ENABLED '
    xINTCON_Save = INTCON       '
    INTCONbits_GIE = 0          '
    #endif                      '

$endif

'-------------------------------------------------------------------------------    
' This new command restore the GIE global interrupt.
$ifndef GIE_Restore
    $define GIE_Restore()       '
    #ifdef __INTERRUPTS_ENABLED '
    If xINTCON_Save.7 = 1 Then  '
        INTCONbits_GIE = 1      '
    EndIf                       '
    #endif                      '
    
$endif
$endif  ' ifenabled
'*******************************************************************************
' Convert a packed BCD (Binary Coded Decimal) value to a decimal value
' Input     : pValue holds the BCD value to convert to decimal
' Output    : pResult holds the decimal value
' Notes     : None
'
$define DS1307_BCDToDec(pValue, pResult) '
    DS1307_bValue = pValue               '
    GoSub _DS1307_BCDToDec               '
    pResult = DS1307_bResult
    
_DS1307_BCDToDec:
    DS1307_bResult = (DS1307_bValue >> 4) * 10
    DS1307_bResult = DS1307_bResult + (DS1307_bValue & $0F)
    Return

'----------------------------------------------------------------------------
' Convert a decimal value to packed BCD (Binary Coded Decimal) value
' Input     : pValue holds the decimal value to convert to BCD
' Output    : pResult holds the packed BCD value
' Notes     : None
'
$define DS1307_DecToBCD(pValue, pResult) '
    DS1307_bValue = pValue               '
    GoSub _DS1307_DecToBCD               '
    pResult = DS1307_bResult

_DS1307_DecToBCD:
    DS1307_bResult = ((DS1307_bValue / 10) << 4)
    DS1307_bResult = DS1307_bResult | (DS1307_bValue // 10)
    Return

'-------------------------------------------------------------------------------
' Read a DS1307 register ($00..$3F)
' Input     : pAddress holds the address of the register
' Output    : Return_Var holds the byte read from the register
' Notes     : None
'
'$define DS1307_ReadRegister(pAddress, pResult) HBusIn cRTC_DEV_ADDR, pAddress,[pResult]

    Dim xDS_Address As LVSByte1
    Dim xDS_Data    As LVSByte2

$define DS1307_ReadRegister(pAddress) _mDS1307ReadRegister pAddress

_mDS1307ReadRegister Macro- pAddress
    #if (Prm_Count > 1)
	   #error "_mDS1307ReadRegister - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mDS1307ReadRegister - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8)
                #error "_mDS1307ReadRegister - (Param1) Should be a Num8"
                Exitm
            #else
                #if (pAddress > 0x38)
                    #error "_mDS1307ReadRegister - (Param1) Should be < $3F"
                    Exitm                
                #else
                    #if(Prm_1 == Num8)
                        Num_Byte pAddress, xDS_Address                    
                    #endif
                #endif
            #endif          
            GoSub _mDS1307ReadRegister_Sub
            #if (_mDS1307ReadRegister_RETURN != 1)
                #error "_mDS1307ReadRegister - Mandatory return parameter misssing"
                Exitm
            #else
                #if(Return_Type != Byte)
                    #error "_mDS1307ReadRegister - Return variable should be a Byte variable"
                    Exitm
                #else
                    Byte_Byte xDS_Data, Return_Var
                #endif
            #endif       
        #endif
    #endif
Endm

#ifMacro- _mDS1307ReadRegister
_mDS1307ReadRegister_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xDS_DeviceRead, xDS_Address,[xDS_Data]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'----------------------------------------------------------------------------
' Write to a DS1307 register ($00..$3F)
' Input     : pAddress holds the address of the register
'           : pData holds the byte to write to the register
' Output    : pACKReturn holds the Acknowledge (Byte) of the DS1307
' Notes     : None
'
'$define DS1307_WriteRegister(pAddress, pData) HBusOut cRTC_DEV_ADDR, pAddress,[pData]

$define DS1307_WriteRegister(pAddress, pData, pACKReturn) _mDS1307_WriteRegister pAddress, pData, pACKReturn

_mDS1307_WriteRegister Macro- pAddress, pData, pACKReturn
    #if (Prm_Count > 3)
	   #error "_mDS1307ReadRegister - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mDS1307ReadRegister - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8)
                #error "_mDS1307ReadRegister - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pAddress, xDS_Address                    
                #endif
            #endif
            #if(Prm_2 != Num8) && (Prm_2 != Byte)
                #error "_mDS1307ReadRegister - (Param2) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_2 == Num8)
                    Num_Byte pData, xDS_Data                    
                #endif
                #if(Prm_2 == Byte)
                    Byte_Byte pData, xDS_Data                    
                #endif
            #endif
            GoSub _mDS1307_WriteReg_Sub
        #endif
    #endif
Endm

#ifMacro- _mDS1307_WriteRegister
_mDS1307_WriteReg_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xDS_DeviceWrite, xDS_Address,[xDS_Data]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-

'----------------------------------------------------------------------------
' Read the RTC control byte ($07)
' Input     : None
' Output    : pResult holds the byte read from the register
' Notes     : See the control constants cRTC_SQWXXXX above
' Syntax    : Value = DS1307_ReadControl(pACKReturn)
'
'$define DS1307_ReadControl(pResult) pResult = DS1307_ReadRegister(cRTC_CONTROL)

$define DS1307_ReadControl(pResult, pACKReturn) pResult = DS1307_ReadRegister(cRTC_CONTROL, pACKReturn)

'----------------------------------------------------------------------------
' Write the RTC Control byte ($07)
' Input     : pData holds the byte to write to the register
' Output    : None
' Notes     : See the control constants cRTC_SQWXXXX above
'
'$define DS1307_WriteControl(pData) HBusOut cRTC_DEV_ADDR, cRTC_CONTROL,[pData]

$define DS1307_WriteControl(pData, pACKReturn) DS1307_WriteRegister(cRTC_CONTROL, pData, pACKReturn)

'----------------------------------------------------------------------------
' Write a byte to Non-Volatile RAM
' Input     : pAddress holds the Non-Volatile RAM address
'           : pData holds the byte to write to Non-Volatile RAM
' Output    : None
' Notes     : The RAM is physically located at
'           : $08..$3F - this routine uses address $00..$37 (56 bytes)
'
'$define DS1307_WriteByte(pAddress, pData) '
'    If pAddress < cRTC_NVRAM_MAX Then     '
'        DS1307_WriteRegister(pAddress + cRTC_NVRAM, pData) '
'    EndIf

$define DS1307_WriteByteNVRAM(pAddress, pData) '
    If pAddress < cRTC_NVRAM_MAX Then                      '
        DS1307_WriteRegister(pAddress + cRTC_NVRAM, pData) '
    EndIf
    
'----------------------------------------------------------------------------
' Read a byte from Non-Volatile RAM
' Input     : pAddress holds the Non-Volatile RAM address
' Output    : pResult holds the byte read from NV RAM
' Notes     : The RAM is physically located at
'           : $08..$3F - this routine uses address $00..$37 (56 bytes)
'
'$define DS1307_ReadByte(pAddress, pResult) pResult = DS1307_ReadRegister(pAddress + cRTC_NVRAM)

$define DS1307_ReadByteNVRAM(pAddress, pResult)  '
    If pAddress < cRTC_NVRAM_MAX Then                    '
    pResult = DS1307_ReadRegister(pAddress + cRTC_NVRAM) '
    EndIf               

'----------------------------------------------------------------------------
' Read the Time
' Input     : None
' Output    : Variable RTC_Second holds the seconds value (in decimal)
'           : Variable RTC_Minute holds the minutes value (in decimal)
'           : Variable RTC_Hour holds the hours value (in decimal)
' Notes     : None
'
'$define DS1307_ReadTime() GoSub _DS1307_ReadTime

'_DS1307_ReadTime:
'    HBusIn cRTC_DEV_ADDR, cRTC_TIME, [RTC_Second, RTC_Minute, RTC_Hour]

'    DS1307_BCDToDec(RTC_Second, RTC_Second)       ' Convert the Seconds value to decimal
'    DS1307_BCDToDec(RTC_Minute, RTC_Minute)       ' Convert the Minutes value to decimal
'    DS1307_BCDToDec(RTC_Hour, RTC_Hour)           ' Convert the Hours value to decimal
'    Return

$define DS1307_ReadTime() _mDS1307ReadTime 

_mDS1307ReadTime Macro- 
    GoSub _mDS1307ReadTime_Sub
Endm

#ifMacro- _mDS1307ReadTime
_mDS1307ReadTime_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xDS_DeviceWrite, cRTC_TIME,[vRTC_Second, vRTC_Minute, vRTC_Hour]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    DS1307_BCDToDec(vRTC_Second, RTC_Second)       ' Convert the Seconds value to decimal
    DS1307_BCDToDec(vRTC_Minute, RTC_Minute)       ' Convert the Minutes value to decimal
    DS1307_BCDToDec(vRTC_Hour, RTC_Hour)           ' Convert the Hours value to decimal
    Return
#endIfMacro-

'----------------------------------------------------------------------------
' Read the Date
' Input     : None
' Output    : Variable RTC_DayOfWeek holds the week day value (in decimal)
'           : Variable RTC_Day holds the days value (in decimal)
'           : Variable RTC_Month holds the months value (in decimal)
'           : Variable RTC_Year holds the years value (in decimal)
' Notes     : None
'
'$define DS1307_ReadDate() GoSub _DS1307_ReadDate

'_DS1307_ReadDate:
'    HBusIn cRTC_DEV_ADDR, cRTC_WDAY, [RTC_DayOfWeek, RTC_Day, RTC_Month, RTC_Year]

'    DS1307_BCDToDec(RTC_DayOfWeek, RTC_DayOfWeek)     ' Convert the WeekDay value to decimal
'    DS1307_BCDToDec(RTC_Day, RTC_Day)                 ' Convert the Days value to decimal
'    DS1307_BCDToDec(RTC_Month, RTC_Month)             ' Convert the Months value to decimal
'    DS1307_BCDToDec(RTC_Year, RTC_Year)               ' Convert the Years value to decimal
'    Return

$define DS1307_ReadDate() _mDS1307ReadDate 

_mDS1307ReadDate Macro- 
    GoSub _mDS1307ReadDate_Sub
Endm

#ifMacro- _mDS1307ReadDate
_mDS1307ReadDate_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xDS_DeviceRead, cRTC_WDAY,[vRTC_DayOfWeek, vRTC_Day, vRTC_Month, vRTC_Year]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    DS1307_BCDToDec(vRTC_DayOfWeek, RTC_DayOfWeek)     ' Convert the WeekDay value to decimal
    DS1307_BCDToDec(vRTC_Day, RTC_Day)                 ' Convert the Days value to decimal
    DS1307_BCDToDec(vRTC_Month, RTC_Month)           ' Convert the Months value to decimal
    DS1307_BCDToDec(vRTC_Year, RTC_Year)               ' Convert the Years value to decimal
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'----------------------------------------------------------------------------
' Set the Time
' Input     : pHour holds the hours value (in decimal)
'           : pMinute holds the minutes value (in decimal)
'           : pSecond holds the seconds value (in decimal)
' Output    : None
' Notes     : None
'
'$define DS1307_WriteTime(pHour, pMinute, pSecond) '
'    RTC_Hour = pHour                              '
'    RTC_Minute = pMinute                          '
'    RTC_Second = pSecond                          '
'    GoSub _DS1307_WriteTime

'_DS1307_WriteTime:
'    DS1307_DecToBCD(RTC_Second, RTC_Second)       ' Convert the Seconds value to decimal
'    DS1307_DecToBCD(RTC_Minute, RTC_Minute)       ' Convert the Minutes value to decimal
'    DS1307_DecToBCD(RTC_Hour, RTC_Hour)           ' Convert the Hours value to decimal
    
'    HBusOut cRTC_DEV_ADDR, cRTC_TIME, [RTC_Second, RTC_Minute, RTC_Hour]
'    Return

$define DS1307_WriteTime(pHour, pMinute, pSecond, pACKReturn) '
    _mDS1307_WriteTime pHour, pMinute, pSecond, pACKReturn

_mDS1307_WriteTime Macro- pHour, pMinute, pSecond, pACKReturn
    #if (Prm_Count > 4)
	   #error "_mDS1307_WriteTime - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 3)
            #error "_mDS1307_WriteTime - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8)
                #error "_mDS1307_WriteTime - (Param1) Should be a Num8"
                Exitm
            #else
                #if (pHour < 24)
                    Num_Byte pHour, vRTC_Hour
                #else
                    #error "_mDS1307_WriteTime - (Param1) Should be a 0 to 23"
                #endif                    
            #endif
            #if(Prm_2 != Num8)
                #error "_mDS1307_WriteTime - (Param2) Should be a Num8"
                Exitm
            #else
                #if (pMinute < 60)
                    Num_Byte pMinute, vRTC_Minute
                #else
                    #error "_mDS1307_WriteTime - (Param2) Should be 0 to 59"
                #endif                    
            #endif
            #if(Prm_3 != Num8)
                #error "_mDS1307_WriteTime - (Param3) Should be a Num8"
                Exitm
            #else
                #if (pSecond < 60)
                    Num_Byte pSecond, vRTC_Second
                #else
                    #error "_mDS1307_WriteTime - (Param4) Should be 0 To 59"
                #endif                    
            #endif
            GoSub _mDS1307_WriteTime_Sub
        #endif
    #endif                  
Endm

#ifMacro- _mDS1307_WriteTime
_mDS1307_WriteTime_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    DS1307_DecToBCD(vRTC_Second, vRTC_Second)       ' Convert the Seconds value to decimal
    DS1307_DecToBCD(vRTC_Minute, vRTC_Minute)       ' Convert the Minutes value to decimal
    DS1307_DecToBCD(vRTC_Hour, vRTC_Hour)           ' Convert the Hours value to decimal
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xDS_DeviceWrite, cRTC_TIME,[vRTC_Second, vRTC_Minute, vRTC_Hour]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'----------------------------------------------------------------------------
' Set the Date
' Input     : pDayOfWeek holds the week day value (in decimal)
'           : pDay holds the days value (in decimal)
'           : pMonth holds the months value (in decimal)
'           : pYear holds the years value (in decimal)
' Output    : None
' Notes     : None
'
'$define DS1307_WriteDate(pDayOfWeek, pDay, pMonth, pYear) '
'    RTC_DayOfWeek = pDayOfWeek                            '
'    RTC_Day = pDay                                        '
'    RTC_Month = pMonth                                    '
'    RTC_Year = pYear                                      '
'    GoSub _DS1307_WriteDate
   
'_DS1307_WriteDate: 
'    DS1307_DecToBCD(RTC_DayOfWeek, RTC_DayOfWeek)     ' Convert the WeekDay value to decimal
'    DS1307_DecToBCD(RTC_Day, RTC_Day)                 ' Convert the Days value to decimal
'    DS1307_DecToBCD(RTC_Month, RTC_Month)             ' Convert the Months value to decimal
'    DS1307_DecToBCD(RTC_Year, RTC_Year)               ' Convert the Years value to decimal
    
'    HBusOut cRTC_DEV_ADDR, cRTC_WDAY, [RTC_DayOfWeek, RTC_Day, RTC_Month, RTC_Year]
'    Return

$define DS1307_WriteDate(pDayOfWeek, pDay, pMonth, pYear, pACKReturn) '
    _mDS1307_WriteDate pDayOfWeek, pDay, pMonth, pYear, pACKReturn


_mDS1307_WriteDate Macro- pDayOfWeek, pDay, pMonth, pYear, pACKReturn
    #if (Prm_Count > 5)
	   #error "_mDS1307_WriteDate - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 4)
            #error "_mDS1307_WriteDate - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8)
                #error "_mDS1307_WriteDate - (Param1) Should be a Num8"
                Exitm
            #else
                #if ((pDayOfWeek > 0) && (pDayOfWeek < 8))
                    Num_Byte pDayOfWeek, vRTC_DayOfWeek
                #else
                    #error "_mDS1307_WriteDate - (Param1) Should be a 1 to 7"
                #endif                    
            #endif
            #if(Prm_2 != Num8)
                #error "_mDS1307_WriteDate - (Param2) Should be a Num8"
                Exitm
            #else
                #if ((pDay > 0) && (pDay < 32))
                    Num_Byte pDay, vRTC_Day
                #else
                    #error "_mDS1307_WriteDate - (Param2) Should be a 1 to 31"
                #endif                    
            #endif
            #if(Prm_3 != Num8)
                #error "_mDS1307_WriteDate - (Param3) Should be a Num8"
                Exitm
            #else
                #if ((pMonth > 0) && (pMonth < 13))
                    Num_Byte pMonth, vRTC_Month
                #else
                    #error "_mDS1307_WriteDate - (Param4) Should be a 1 to 12"
                #endif                    
            #endif
            #if(Prm_4 != Num8)
                #error "_mDS1307_WriteDate - (Param4) Should be a Num8"
                Exitm
            #else
                Num_Byte pYear, vRTC_Year
            #endif
            GoSub _mDS1307_WriteDate_Sub
        #endif
    #endif                  
Endm

#ifMacro- _mDS1307_WriteDate
_mDS1307_WriteDate_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    DS1307_DecToBCD(vRTC_DayOfWeek, vRTC_DayOfWeek)     ' Convert the WeekDay value to BCD
    DS1307_DecToBCD(vRTC_Day, vRTC_Day)                 ' Convert the Days value to BCD
    DS1307_DecToBCD(vRTC_Month, vRTC_Month)             ' Convert the Months value to BCD
    DS1307_DecToBCD(vRTC_Year, vRTC_Year)               ' Convert the Years value to BCD
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xDS_DeviceWrite, cRTC_WDAY,[vRTC_Second, vRTC_Minute, vRTC_Hour]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'----------------------------------------------------------------------------
' Return the name of the day
' Input     : pDayOfWeek holds the day value 1 to 7 (Sunday to Saturday)
' Output    : RTC_DayName holds a string containing the day's name
' Notes     : None
'
$define DS1307_Get_DayName(pDayOfWeek, pResult) '
    DS1307_bValue = pDayOfWeek                  '
    _DS1307_Get_DayName                         '
    pResult = RTC_DayName

_DS1307_Get_DayName Macro-
    GoSub __DS1307_Get_DayName
Endm

#ifMacro- _DS1307_Get_DayName
__DS1307_Get_DayName:
$ifdef _English_
    Select DS1307_bValue
        Case 2
            RTC_DayName = "Monday"
        Case 3
            RTC_DayName = "Tuesday"
        Case 4
            RTC_DayName = "Wednesday"
        Case 5
            RTC_DayName = "Thursday"
        Case 6
            RTC_DayName = "Friday"
        Case 7
            RTC_DayName = "Saturday"
        Case 1
            RTC_DayName = "Sunday"
    EndSelect
$endif
$ifdef _French_
    Select DS1307_bValue
        Case 2
            RTC_DayName = "Lundi"
        Case 3
            RTC_DayName = "Mardi"
        Case 4
            RTC_DayName = "Mercredi"
        Case 5
            RTC_DayName = "Jeudi"
        Case 6
            RTC_DayName = "Vendredi"
        Case 7
            RTC_DayName = "Samedi"
        Case 1
            RTC_DayName = "Dimanche"
    EndSelect
$endif
$ifdef _Spanish_
    Select DS1307_bValue
        Case 2
            RTC_DayName = "Lunes"
        Case 3
            RTC_DayName = "Martes"
        Case 4
            RTC_DayName = "Miercoles"
        Case 5
            RTC_DayName = "Jueves"
        Case 6
            RTC_DayName = "Viernes"
        Case 7
            RTC_DayName = "Sabado"
        Case 1
            RTC_DayName = "Domingo"
    EndSelect
$endif
    Return
#endIfMacro

'----------------------------------------------------------------------------
' Return the name of the month
' Input     : pMonth holds the month value 1 to 12 (January to December)
' Output    : RTC_MonthName holds a string containing the month's name
' Notes     : None
'
$define DS1307_Get_MonthName(pMonth, pResult) '
    DS1307_bValue = pMonth                    '
    _DS1307_Get_MonthName                     '
    pResult = RTC_MonthName

_DS1307_Get_MonthName Macro-
    GoSub __DS1307_Get_MonthName
Endm

#ifMacro- _DS1307_Get_MonthName
__DS1307_Get_MonthName:
$ifdef _English_
    Select DS1307_bValue
        Case 1
            RTC_MonthName = "January"
        Case 2
            RTC_MonthName = "February"
        Case 3
            RTC_MonthName = "March"
        Case 4
            RTC_MonthName = "April"
        Case 5
            RTC_MonthName = "May"
        Case 6
            RTC_MonthName = "June"
        Case 7
            RTC_MonthName = "July"
        Case 8
            RTC_MonthName = "August"
        Case 9
            RTC_MonthName = "September"
        Case 10
            RTC_MonthName = "October"
        Case 11
            RTC_MonthName = "November"
        Case 12
            RTC_MonthName = "December"
    EndSelect
$endif
$ifdef _French_
    Select DS1307_bValue
        Case 1
            RTC_MonthName = "Janvier"
        Case 2
            RTC_MonthName = "Fevrier"
        Case 3
            RTC_MonthName = "Mars"
        Case 4
            RTC_MonthName = "Avril"
        Case 5
            RTC_MonthName = "Mai"
        Case 6
            RTC_MonthName = "Juin"
        Case 7
            RTC_MonthName = "Juillet"
        Case 8
            RTC_MonthName = "Aout"
        Case 9
            RTC_MonthName = "Septembre"
        Case 10
            RTC_MonthName = "Octobre"
        Case 11
            RTC_MonthName = "Novembre"
        Case 12
            RTC_MonthName = "Decembre"
    EndSelect
$endif
$ifdef _Spanish_
    Select DS1307_bValue
        Case 1
            RTC_MonthName = "Enero"
        Case 2
            RTC_MonthName = "Febrero"
        Case 3
            RTC_MonthName = "Marzo"
        Case 4
            RTC_MonthName = "Abril"
        Case 5
            RTC_MonthName = "Mayo"
        Case 6
            RTC_MonthName = "Junio"
        Case 7
            RTC_MonthName = "Julio"
        Case 8
            RTC_MonthName = "Augusto"
        Case 9
            RTC_MonthName = "Septiembre"
        Case 10
            RTC_MonthName = "Octubre"
        Case 11
            RTC_MonthName = "Noviembre"
        Case 12
            RTC_MonthName = "Diciembre"
    EndSelect
$endif
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Compress the data of the DS1307 RTC to send to the 24LC256,
' Saving byte6 & Byte7 to write 2 data bytes like temperature and Humidity parameters.
' Input     : DS1307_Array + 2 external variables (pData1, pData2)
' Output    : E2PArray
' Notes     : These Macros could be used to write the DS1307 info to an external eeprom memory (24LC256)
'           : Use the "24LC256.inc" library downloaded from the WIKI of the PDS forum.
'
$define DS1307_ZipPage8(pData1, pData2) '
    _mDS1307_ZIPPage8                   '
    E2PArray#6 = pData1                 '
    E2PArray#7 = pData2
    
_mDS1307_ZIPPage8 Macro-
    GoSub _mDS1307_ZIPPage8_Sub
Endm

#ifMacro- _mDS1307_ZIPPage8
_mDS1307_ZIPPage8_Sub:
    E2PArray = DS1307_Array                         ' Copy the DS1307 array.
    E2PArray#3 = E2PArray#3 | (E2PArray#5 << 4)     ' Save Low nibble Byte 5 to Byte 3 High nibble. 
    E2PArray#5 = E2PArray#6                         ' Save Byte 6 to Byte 5. Now the Byte 6 is free.
    ' After this macro the 2 datas will be loaded to this array.
    Return
#endIfMacro-

$define DS1307_UnZipPage8(pReturnData1, pReturnData2) '
    pReturnData1 =  E2PArray#6                        '
    pReturnData2 =  E2PArray#7                        '
    _mDS1307_UnZipPage8

_mDS1307_UnZipPage8 Macro-
    GoSub _mDS1307_UnZipPage8_Sub
Endm

#ifMacro- _mDS1307_UnZipPage8
_mDS1307_UnZipPage8_Sub:
    ' Before this macro the 2 datas are restored.
    E2PArray#6 = E2PArray#5               ' Restore Byte 5 to position Byte 6
    E2PArray#5 = (E2PArray#3 >> 4) & $0F  ' Restore the Byte 3 high nibble to the Byte 5 low nibble.
    E2PArray#3 = E2PArray#3 & $07         ' Clean the Byte 3 High nibble.
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Detect the DS1307
' Input     : None
' Output    : pACKReturn holds 0 if the DS1307 is available
' Notes     : Uses the write/Read the NVRAM for confirmation
'           : This procedure assumes that there is only 1 I2C device attached to the bus
'
$define DS1307_Present(pACKReturn)    '
    DS1307_WriteByteNVRAM(8, $A5)     '
    DS1307_ReadByteNVRAM(8, xDS_Data) '
    pACKReturn = xDS_Data ^ $A5

'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
$ifdef _HRSOutVersionFile_inc_
_DS1307_INC_Version: CData "DS1307-S2.inc Version 1.0 - 15/03/2016",0
$endif
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
_DS1307_INCEXIT:
'===============================================================================
' Set the I2C pins to digital. (Result = 2 lines)
$if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22) Or (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20)
$ifndef _AnselI2CSoftPins_
$define _AnselI2CSoftPins_
#if (__SCL_PORT == PORTA)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTB)
    #if (__SCL_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SCL_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SCL_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SCL_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SCL_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SCL_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SCL_PORT == PORTE)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
'-------------------------------------------------------------------------------
#if (__SDA_PORT == PORTA)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTB)
    #if (__SDA_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SDA_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SDA_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SDA_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SDA_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SDA_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SDA_PORT == PORTE)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
$endif      ' _AnselI2CSoftPins_
$endif      ' ifdevice
'===============================================================================
$endif      ' _DS1307_INC_
'===============================================================================

