'****************************************************************
'*  Name    : LCD_AMI18_Board.inc                               *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 15/03/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************

$ifndef _LCD_AMI18_Board_inc_
$define _LCD_AMI18_Board_inc_

    GoTo _OverLCD_AMI18_Board
'===============================================================================
'===============================================================================
' Declaration of the AMI18 LCD SHIELD pins
'*******************************************************************************
'-------------------------------------------------------------------------------
' Add LOCAL (non-persistent) variables here
$ifndef _LVBS_Block1_
    $define _LVBS_Block1_
    Dim LVSByte1 As Byte System
    Dim LVSByte2 As Byte System
    Dim LVSByte3 As Byte System
    Dim LVSByte4 As Byte System
$endif

' Definicin de los PORTS del proyecto
' Notes   : 
' 
    Symbol SW_1 = PORTA.0          ' Pin where S1 is connected 
    Symbol SW_2 = PORTA.1          ' Pin where S2 is connected
    Symbol SW_3 = PORTA.2          ' Pin where S3 is connected
    Symbol SW_4 = PORTA.3          ' Pin where S4 is connected
    Symbol PulseInput = PORTA.3    ' Pin where SOUT/DS1307 is connected
    Symbol LCD_Backlight = LATB.1
    Symbol Buzzer = LATB.0         ' Pin where the buzzer is connected

    ' Declare the I2C pins
'    Symbol SDAPin = PORTC.4
'    Symbol SCLPin = PORTC.3
'------------------------------------------------------------------------------- 
    ' Virtual PORT for all Buttons.
    Dim xPORTA_Temp As Byte System

    Dim Button1 As xPORTA_Temp.0
    Dim Button2 As xPORTA_Temp.1
    Dim Button3 As xPORTA_Temp.2
    Dim Button4 As xPORTA_Temp.3  

'===============================================================================
'===============================================================================
' Definiton of the Buttons PORT
' Input   : none
' Output  : none 
' Sintaxis: DEF_Buttons()
' Notes   : 

$define DEF_Buttons() _mDEF_Buttons

_mDEF_Buttons Macro-
    GoSub _mDEF_Buttons_Sub
Endm

#ifMacro- _mDEF_Buttons,_mRead_Buttons,_mWait_ButtonsOff
_mDEF_Buttons_Sub:
    Input PORTA.0       ' PORTA.0  Pin where S1 is connected 
    Input PORTA.1       ' PORTA.1  Pin where S2 is connected
    Input PORTA.2       ' PORTA.2  Pin where S3 is connected
    Input PORTA.3       ' PORTA.3  Pin where S4 is connected
    Return
#endIfMacro-     
'-------------------------------------------------------------------------------
' Read the buttons of the LCD board.
' Input   : 
' Output  : xPORTA_Temp 
' Sintaxis: Var1 = Read_Buttons(), Read_Buttons()
' Notes   : 
' Modification: 10/10/2013 Write more security inside the macro.
$define Read_Buttons() _mRead_Buttons

_mRead_Buttons Macro-
    GoSub _mRead_Buttons_Sub
    #if(_mRead_Buttons_Return == 1)
        #if(Return_Type != Byte)
            #error "_mRead_Buttons - Return variable should be a Byte"
            Exitm
        #else 
            Byte_Byte xPORTA_Temp, Return_Var
        #endif
    #endif
Endm

#ifMacro- _mRead_Buttons
_mRead_Buttons_Sub:
    GoSub _mDEF_Buttons_Sub     ' Recall PortA definition. More security.
'    xPORTA_Temp = ~PORTA & %00001111
' Same and more faster
Asm
    Comf PORTA,W,0
    Andlw 15
    Movwf xPORTA_Temp,0
EndAsm
    Return
#endIfMacro-    
'-------------------------------------------------------------------------------
' This routine check the buttons And Wait Until the buttons are NOT pressed.
' Input : PORTA
' Output: none
' Syntax: Wait_ButtonsOff()
' Notes : 
'
$define Wait_ButtonsOff() _mWait_ButtonsOff 

_mWait_ButtonsOff Macro-
    GoSub _mWait_ButtonsOff_Sub
Endm

#ifMacro- _mWait_ButtonsOff
_mWait_ButtonsOff_Sub:
    GoSub _mDEF_Buttons_Sub     ' Recall PortA definition. More security.

    Repeat
'        xPORTA_Temp = ~PORTA & %00001111
        ' Same and more faster
Asm
        Comf PORTA,W,0
        Andlw 15
        Movwf xPORTA_Temp,0
EndAsm        
        DelayMS 40
    Until xPORTA_Temp = $00
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Alternative To the standard Proton Cls command
' Input  : 
' Output :  
' Syntax : CLL {Line, {Pos}} Command
' Notes  : For deleting some characters on the LCD.
'
' For users still using an alphanumeric LCD this provides a useful enhanced
' alternative to the standard Proton CLS command.  When invoked with no parameters
' it will perform a standard Proton CLS, however, if you add a line number argument
' it will clear just that line, if you add a second argument (Position) it will clear
' from that position to the end of the line leaving the cursor at the beginning of the clear.
'
' CLL {Line, {Pos}} Command
'
'      With no arguments same as proton CLS
'
'      With 1 argument (Line), clears line specified in argument
'      Con 1 argumento (Linea), borra la lnea especificada en el argumento
'
'      With 2 arguments (Line, Pos), clears to end of Line from Position
'      Con 2 argumentos (Linea, Posicin), borra el final de la lnea desde el argumento Posicin
'
$ifndef CLL
    Dim xCUR_Row As LVSByte1
    Dim xCUR_Col As LVSByte2
    Dim xCUR_Chrs As LVSByte3
    
$define CLL(pLine, pPos) _mCLL pLine, pPos
 
_mCLL Macro- pLine, pPos
    #if (Prm_Count >  2)
        #error "_mCLL - Too many parameters"
    #else
        #if (Prm_Count == 0)
'      Add additional code for no params here .......
        Cls    ' Here is the additional code added after the generator has produced the outline
        #else
            ' 1 additional character: Delete the line x. (all characters)
            #if (Prm_Count == 1)
                #if (Prm_1 != Byte) && (Prm_1 != Num8)
                    #error "_mCLL - Line(Param 1) should be a Byte variable or number"
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pLine, xCUR_Row
                #endif
                #if (Prm_1 == Num8)
                    Num_Byte pLine, xCUR_Row
                #endif
                GoSub _mCLL_Sub1
            #endif

            ' 2 additional characters: Delete the line x from position y.
            #if (Prm_Count == 2)
                #if (Prm_1 != Byte) && (Prm_1 != Num8)
                    #error "_mCLL - Line(Param 1) should be a Byte variable or number"
                #endif
                #if (Prm_1 == Byte)
                    Byte_Byte pLine, xCUR_Row
                #endif
                #if (Prm_1 == Num8)
                    Num_Byte pLine, xCUR_Row
                #endif

                #if (Prm_2 != Byte) && (Prm_2 != Num8)
                    #error "_mCLL - Pos(Param 2) should be a Byte variable or number"
                #endif
                #if (Prm_2 == Byte)
                    Byte_Byte pPos, xCUR_Col
                #endif
                #if (Prm_2 == Num8)
                    Num_Byte pPos, xCUR_Col
                #endif
                GoSub _mCLL_Sub2
            #endif

        #endif
    #endif
Endm

#ifMacro- _mCLL
_mCLL_Sub1:
    xCUR_Col = 1                ' The Column most be > 0.

_mCLL_Sub2:
    Cursor xCUR_Row, xCUR_Col 
    xCUR_Chrs = __LCD_CharsLenght - xCUR_Col
    Inc xCUR_Chrs
    Repeat
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Print $20               ' Send a SPACE to the LCD
        Dec xCUR_Chrs           ' More character to delete.
    Until xCUR_Chrs = 0         ' Until the last number of characters deleted
    Cursor xCUR_Row, xCUR_Col   ' Set the Cursor to the Origin position.
    Return
#endIfMacro-
$endif
'Note  This macro requires a constant __LCD_NoCharacters to be defined to reflect your
' hardware configuration.  E.g. Symbol __LCD_NoCharacters = 16 for a 16 character display.  
'-------------------------------------------------------------------------------
' Send a sound the Buzzer (Beep(Delay)) ceramic or standard piece.
' Input   : Number from 1 to 255
' Output  : none
' Syntax  : Beep(Delay)
' Notes   : Set the buzzer
' The precision of the time  and frequency could be around +-10%
' For Sound Buzzer,[120,BeepDelay] 
'
' Temp Variable
    Dim BeepDelay As LVSByte1

$define Beep(pValue) mBeep pValue

mBeep Macro pValue
	#if(Prm_Count != 1)         
        ' Ensure the correct amount of parameters
		#error "Incorrect amount of parameters for mBeep macro. 1 Parameter required!"	
		Exitm
    #else
        #if (Prm_1 != Num8)
            ' Ensure the parameter is a 8 bits number
            #error "mBeep Macro - (Param1) Should be a Num8"
            Exitm
        #else    
            #if (Prm_1 == Num8)
                #if(pValue == 0)
                    #error "mBeep Macro - (Param1) Should be > 0"
                    Exitm
                #endif
                Num_Byte pValue, BeepDelay
            #endif            
            GoSub _mBeep_Sub
        #endif
    #endif
Endm

'#ifdef mBeep#REQ
#ifMacro- mBeep
_mBeep_Sub:
$ifdef _BuzzerCeramic_
     BeepDelay = BeepDelay / 11
     Sound Buzzer,[120,BeepDelay]   ' Frequency Buzzer is +-1250hZ, and delay +-100mS
     Low Buzzer
$else
     High Buzzer
     DelayMS BeepDelay                          
     Low Buzzer
$endif
    Return
#endIfMacro-
'-------------------------------------------------------------------------------


'===============================================================================
$ifdef _HRSOutVersionFile_inc_
_LCD_AMI18_Board_inc_Version: CData "LCD_AMI18_Board.inc Version 1.0 - 15/03/2016",0
$endif
'===============================================================================
_OverLCD_AMI18_Board:
'-------------------------------------------------------------------------------
    ' Declare All_Digital = On   
    ADCON0 = 0
    ADCON1 = 0
    ADCON2 = 0    
    CM1CON0 = 0
    CM2CON0 = 0
    CM2CON1 = 0    
    $if _device = _18F25K20
    ANSEL = 0
    ANSELH = 0
    $endif
    $if _device = _18F25K22
    ANSELA = 0
    ANSELB = 0
    ANSELC = 0
    $endif

' TRIS all pins
    Output PORTB        ' LCD port.
    Input PORTA.0       ' PORTA.0  Pin where S1 is connected 
    Input PORTA.1       ' PORTA.1  Pin where S2 is connected
    Input PORTA.2       ' PORTA.2  Pin where S3 is connected
    Input PORTA.3       ' PORTA.3  Pin where S4 is connected    
    Input PulseInput    ' Pin SOUT/DS1307
    Output LCD_Backlight
    Output Buzzer
   
'===============================================================================
$endif
'-------------------------------------------------------------------------------
